package org.jabricks.templates;

import java.awt.*;

import javax.swing.*;
import javax.swing.border.Border;
// import javax.swing.table.TableColumnModel;

import org.jabricks.widgets.grid.Grid;
import org.jabricks.widgets.grid.IGrid;
import org.jabricks.jdialog.ComponentParams;
import org.jabricks.jdialog.JBDialog;
import org.osgi.framework.BundleContext;
import org.jabricks.resources.IResources;
import org.jabricks.resources.ResourcesImpl;

import java.util.Locale;
import java.util.Properties;
import java.util.ResourceBundle;

import org.jabricks.widgets.constants.IWidgetsConstants;

// import org.jabricks.basegui.common.JTableColumnModelListener;

public class JDialogForm extends JBDialog implements IWidgetsConstants
{
	private static final long  serialVersionUID = 1L;
	
	private  int            DEFAULT_WIDTH  = 420;
	private  int            DEFAULT_HEIGHT = 315;

	public   JButton        btnOk          = null;
	public   JButton        btnClose       = null;
    
	private  final  String  RSC_btn_ok     = "btn.ok"   ;
	private  final  String  RSC_btn_cancel = "btn.close";

	private  JSplitPane     split          = null;
	private  final  int     DIVIDER        = 300;	
	private  final  String  SPLITPANE      = "SPLITPANE";

	private  IGrid          grid           = null;
	private  final  String  GRIDNAME       = "Grid";

	// типы значений в колонках таблицы
	private Class<?>[] column_classes = {
                            String.class, Integer.class,
                            Double.class, Double.class};
	// наименование колонок таблицы
	private String [] column_names = {"name", "quantity",
	                                  "price", "cost"};
	private int[]     column_width = {220, 80, 90, 90};
	// выравнивание значений в ячейках таблицы
	private Integer[] column_align = {
			                   JLabel.LEFT, JLabel.RIGHT,
	                           JLabel.RIGHT,JLabel.RIGHT};

	private ResourceBundle rsc = null;
	private String         RSC = "properties/properties";
    //-----------------------------------------------------
	public JDialogForm(final BundleContext context)
	{
		super(context);
		
        IResources rsc = new ResourcesImpl();
        // Определение иконки
       	if (rsc != null)
       		setIconImage(rsc.getAppIcon());

        // Чтение регулируемых настроек
        readComponentParams();

        if (form_dim != null) {
            setSize(form_dim);
            DEFAULT_WIDTH  = form_dim.width;
            DEFAULT_HEIGHT = form_dim.height;
        }
        setPreferredSize(new Dimension(DEFAULT_WIDTH, 
                                       DEFAULT_HEIGHT));
		// Создание интерфейса GUI
		createGUI();
        // Получение панели содержимого
        Container container = getContentPane();
        
        // Размещение панели desk
		container.add(pnlDesk, BorderLayout.CENTER);
        // Размещение панели с кнопками
        container.add(createButtons(), BorderLayout.SOUTH);        
		// Центрирование формы
		formCentering(DEFAULT_WIDTH, DEFAULT_HEIGHT);
       	// Расположение поверх остальных
		setAlwaysOnTop(true);
	}
    //-----------------------------------------------------
	/**
	 * Создание интерфейса рабочей области формы
	 * @param locale текущая локализация.
	 * @return панели управления JPanel
	 */
    private void createGUI()
    {
    	// Создание рабочей области
    	super.createDesk();
		Border border;
		border = BorderFactory.createEmptyBorder(2,2,2,2);
    	pnlDesk.setBorder (border);

		split = new JSplitPane();
		split.setName(SPLITPANE);
        
		split.setOneTouchExpandable(false);
		split.setOrientation(JSplitPane.HORIZONTAL_SPLIT);
		
		if (cparams.size() == 0)
			split.setDividerLocation(DIVIDER);
		else {
			// Определение размеров колонок
			ComponentParams cp;
			String          cls;
			cls = JSplitPane.class.getSimpleName();
			cp = extractComponentParams(cls, SPLITPANE);
			if (cp != null)
				split.setDividerLocation(cp.getLocation());
			else
				split.setDividerLocation(DIVIDER);
		}
		
		pnlDesk.setLayout(new BorderLayout());
		pnlDesk.add(split, BorderLayout.CENTER);
		
		JPanel pnlRight = new JPanel();
		
		pnlRight.setBackground(Color.green);
		
		createGrid();
		split.setLeftComponent (grid.asWidget());		
		split.setRightComponent(pnlRight);
    }
    //-----------------------------------------------------
	/**
	 * Создание панели кнопок управления
	 * @return панели управления JPanel
	 */
    private JPanel createButtons()
    {
        String caption = EMPTY;
        // Кнопка ОК в панель
        btnOk = new JButton(caption);
        // Кнопка Закрыть
        btnClose = new JButton(caption);

        Dimension dim = new Dimension(BUTTON_width,
        		                      BUTTON_height);
        btnOk   .setPreferredSize(dim);
        btnClose.setPreferredSize(dim);
        
        /*
         *  Создание панели 'pnlButtons' с последовательным
         *  расположением компонентов и выравниванием по 
         *  правому краю 'flow'
         */
        JPanel flow = super.createButtonsPanel(); 
        pnlButtons.add (btnOk   );
        pnlButtons.add (btnClose);
        
        changeLocale (locale);

        return flow;
    }
    //-----------------------------------------------------
    /**
     * Процедура локализации интерфейса
     * @param locale объект локализации
     */
    public void changeLocale (Locale locale)
    {
    	this.locale = locale;
    	
    	rsc = ResourceBundle.getBundle (RSC, locale);
    	
		String ok      = RSC_btn_ok;
		String cancel  = RSC_btn_cancel;
		String caption = RESOURCE_title;
		
		if (rsc != null) {
			caption = rsc.getString(RESOURCE_title);
			ok      = SPAN_normal + rsc.getString(ok    );
			cancel  = SPAN_normal + rsc.getString(cancel);
		}
		this.setTitle(caption);
		btnOk      .setText(ok    );
		btnClose   .setText(cancel);

		Properties props = loadProperties(RSC,locale);
		grid.setLocale(props);
    }
    //-----------------------------------------------------
    private void createGrid()
    {
        // Создание таблицы
        grid = new Grid(column_classes, column_names);
        // Определение наименования таблицы
		grid.getTable().setName(GRIDNAME);
        // Настройка таблицы
        grid.setColumnsWidth     (column_width);
        grid.setColumnsAlign     (column_align);
        grid.setReorderingAllowed(false       );

		if (cparams.size() > 0) {
			// Определение размеров колонок
			ComponentParams cp;
			String          cls;
			cls = JTable.class.getSimpleName();
			cp = extractComponentParams(cls, GRIDNAME);
			if (cp != null) {
				String columns = cp.getColumns();
				String[] cols = columns.split(SEMICOLON);
				int cnt = grid.getTable().getColumnCount();
				if (cnt == cols.length) {
					int[] cw = new int[cols.length];
					try {
						for (int i = 0; i < cols.length; i++)
							cw[i] = Integer.valueOf(cols[i]);
						grid.setColumnsWidth(cw);
					} catch (Exception e) {}
				}
			}
		}
    }
    //---------------------------------------------------------------------------------------------------
}
